#!/usr/bin/python3
# -*- coding: utf-8 -*-
import os

import utils


def vhost_write_cmds(dir_path, name, server_name, **data):
    params = [('{{ server_name }}', server_name)]
    for k, v in data.items():
        params.append(('{{ %s }}' % k, v))
    return [
        dict(
            line='write',
            template='%s/vhost_%s.conf' % (dir_path, name),
            target='/etc/nginx/sites-available/%s.conf' % name,
            params=params,
        ),
        'ln -sfn ../sites-available/%s.conf /etc/nginx/sites-enabled/%s.conf' % (name, name),
        'echo "Vhost %s updated (domain: %s)."' % (name, server_name),
    ]


def setup(interactive=True):
    dir_path = utils.get_dir(__file__)
    cmds = [
        'apt-get install --yes nginx uwsgi uwsgi-plugin-python3',
        '[ -f "/etc/nginx/conf.d/ssl.conf" ] || cp "%s/ssl.conf" "/etc/nginx/conf.d/ssl.conf"' % dir_path,
        '[ -f "/etc/nginx/nginx.conf.back" ] || cp "/etc/nginx/nginx.conf" "/etc/nginx/nginx.conf.back"',
        'cp "%s/nginx.conf" "/etc/nginx/nginx.conf"' % dir_path,
        'cp "%s/mime.types" "/etc/nginx/mime.types"' % dir_path,
        'cp "%s/uwsgi_params" "/etc/nginx/uwsgi_params"' % dir_path,
        'rm -f /etc/nginx/sites-enabled/default',
    ]
    hosts = list()
    # Stats vhost (nginx_status for monitoring)
    cmds.extend(vhost_write_cmds(dir_path, 'stats', 'localhost'))
    # FTP: videos vhost
    if os.path.exists('/var/www/videos'):
        cmds.append('cp %s/crossdomain.xml /var/www/videos/crossdomain.xml' % dir_path)
        server_name = utils.get_conf('ftp_server_name') or 'videos'
        cmds.extend(vhost_write_cmds(dir_path, 'videos', server_name))
        hosts.append(server_name)
    # Wowza: streaming vhost
    if os.path.exists('/var/www/streaming'):
        cmds.append('cp %s/crossdomain.xml /var/www/streaming/crossdomain.xml' % dir_path)
        server_name = utils.get_conf('wowza_server_name') or 'streaming'
        cmds.extend(vhost_write_cmds(dir_path, 'streaming', server_name))
        hosts.append(server_name)
    # MediaServer: mediaserver-msuser vhost
    if os.path.exists('/home/msuser/msinstance'):
        server_name = utils.get_conf('ms_server_name') or 'mediaserver'
        cmds.extend(vhost_write_cmds(dir_path, 'mediaserver-msuser', server_name,
            secret=utils.get_conf('ms_secret') or 'secret', worker_ip=utils.get_conf('celerity_worker_ip') or '127.0.1.1'))
        hosts.append(server_name)
    # Monitor: msmonitor vhost
    if os.path.exists('/home/msmonitor/msmonitor'):
        server_name = utils.get_conf('monitor_server_name') or 'msmonitor'
        cmds.extend(vhost_write_cmds(dir_path, 'msmonitor', server_name))
        hosts.append(server_name)
    # SkyReach: skyreach vhost
    if os.path.exists('/home/skyreach/htdocs'):
        server_name = utils.get_conf('skyreach_server_name') or 'skyreach'
        cmds.extend(vhost_write_cmds(dir_path, 'skyreach', server_name))
        hosts.append(server_name)
    cmds.append('service nginx reload')
    utils.run_commands(cmds)
    # Update hosts file
    rc, hostname = utils.exec_cmd('hostname', get_out=True)
    if rc == 0 and hostname not in hosts:
        hosts.insert(0, hostname)
    with open('/etc/hosts', 'r') as fo:
        content = fo.read()
    new_content = list()
    found_127 = False
    for line in content.split('\n'):
        if not found_127 and line.startswith('127.0.0.1'):
            found_127 = True
            for host in hosts:
                if ' ' + host not in line:
                    line += ' ' + host
                    utils.log('Adding host %s to /etc/hosts 127.0.0.1 aliases.' % host)
        new_content.append(line)
    if not found_127:
        new_content.append('127.0.0.1 %s' % ' '.join(hosts))
    new_content = '\n'.join(new_content)
    if new_content != content:
        with open('/etc/hosts', 'w') as fo:
            fo.write(new_content)
        utils.log('/etc/hosts updated.')
